import { EOChildren } from 'mve-core/childrenBuilder';
import { mve } from 'mve-core/util';
/**
 * 创建视图
 * @param me
 * @param router
 * @param createRoutes
 * @param other
 * @returns
 */
export declare function routerView<T>(me: mve.LifeModel, router: Router<T>, createRoutes: {
    [key in keyof T]: CreateSubRouter<T[key]>;
}, other?: {
    notFoundView?: Router<any>;
}): import("mve-core/childrenBuilder").EOChildFun<Node>;
export interface QueryParam {
    [key: string]: string | string[];
}
export interface RouterParam {
    path: string[];
    query: QueryParam;
}
export declare type CreateSubRouter<T> = (parentPath: string[]) => Router<T>;
export declare class Router<T> {
    readonly parentPath: string[];
    private fun;
    readonly param: mve.Value<RouterParam>;
    constructor(parentPath: string[], fun: (me: mve.LifeModel, router: Router<T>) => EOChildren<Node>);
    current(): keyof T;
    render(me: mve.LifeModel): EOChildren<Node>;
    /**
     * 生成合适的路由
     * @param path
     * @param query
     * @returns
     */
    href<K extends keyof MSCircle<T>>(path: K, query: (MSCircle<T>[K] extends QueryWrapper ? MSCircle<T>[K]['value'] : never)): string;
    /**
     * 中能在自己的下游路径漫游
     * @param path
     * @param query
     */
    go<K extends keyof MSCircle<T>>(path: K, query: (MSCircle<T>[K] extends QueryWrapper ? MSCircle<T>[K]['value'] : never)): void;
}
export declare function queryToString(q?: QueryParam): string;
export declare function createRouter<T>(fun: (me: mve.LifeModel, router: Router<T>) => EOChildren<Node>): CreateSubRouter<T>;
export declare function hashToParam(hash: string): RouterParam;
export declare function locationHashChange(route: Router<any>): void;
export declare function locationHashBind(route: Router<any>): void;
export declare class QueryWrapper<T extends QueryParam = {}> {
    value: T;
    constructor(value: T);
}
export declare type MSCircle<T> = MCircle<T, QueryWrapper, '/'>;
/**
 * 打平
 */
export declare type MCircle<T, V, S extends string> = {
    [k1 in IncludeKey<T, V>]: T[k1];
} & CombineRouterSub<ExcludeKey<T, V> & string, T, S>;
/**子级加前缀 */
export declare type CombineRouterSub<K extends (keyof V & string), V, S extends string> = UnionToIntersection<ToUnion<K, V[K], S>>;
/**加前缀，变联合*/
declare type ToUnion<K extends string, V, S extends string> = V extends {
    [key in infer KV]: any;
} ? KV extends string ? {
    [key2 in `${K}${S}${KV}`]: V[KV];
} : never : never;
/**
 * 联合转交叉
 */
export declare type UnionToIntersection<U> = (U extends any ? (k: U) => void : never) extends ((k: infer I) => void) ? I : never;
/**
* T里选择value的类型是F的key
*/
export declare type IncludeKey<T, F> = {
    [key in keyof T]: T[key] extends F ? key : never;
}[keyof T];
/**
* T里选择value的类型不是F的key
*/
export declare type ExcludeKey<T, F> = {
    [key in keyof T]: T[key] extends F ? never : key;
}[keyof T];
export {};
